unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, 
  Grids, StdCtrls, DirectDisk, ComCtrls, Spin, FileCtrl, ExtCtrls;

type
  TSectorArray = Array[0..(512*4)-1] of Byte;

  TForm1 = class(TForm)
    StringGrid1: TStringGrid;
    DirDisk1: TDirDisk;
    StatusBar1: TStatusBar;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    bReadSec: TButton;
    seFirstSector: TSpinEdit;
    bWriteSec: TButton;
    cbDrive: TDriveComboBox;
    procedure bReadSecClick(Sender: TObject);
    procedure StringGrid1KeyPress(Sender: TObject; var Key: Char);
    procedure FormCreate(Sender: TObject);
    procedure StringGrid1Exit(Sender: TObject);
    procedure bWriteSecClick(Sender: TObject);
    procedure StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure StringGrid1SelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure StringGrid1GetEditText(Sender: TObject; ACol, ARow: Longint;
      var Value: string);
    procedure cbDriveChange(Sender: TObject);
    procedure seFirstSectorChange(Sender: TObject);
  private
    { Private declarations }
    A, B: TSectorArray;
    OldDrive: Char;
    FloppyType: TFloppyType;
    procedure BytesToGrid;
    procedure OnFocusChange;
  public
    { Public declarations }
  end;

const
  CharSellsCount = 17;
var
  ColOld: Integer = 1;
  RowOld: Integer = 1;
  CellKeyPress: Integer;
  SectorLength: Word;
var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.bReadSecClick(Sender: TObject);
var
  Floppy: Boolean;
  OkReadSector: Boolean;
begin
  Floppy := UpCase(cbDrive.Drive) in ['A','B'];
  FloppyType := D1440;
  with DirDisk1 do begin
    if not Floppy then DriveName := cbDrive.Drive;
    if (TotalNumberOfClusters*SectorsPerCluster)>Cardinal(seFirstSector.Value)
    then begin
      FirstSector := seFirstSector.Value;
      Sectors := 1;
      if Floppy then begin
        case DiskSize(Ord(cbDrive.Drive)-Ord('A')+1) of
          362496: FloppyType := D360;
          730112: FloppyType := D720;
          1213952: FloppyType := D1200;
          1457664: FloppyType := D1440;
        else FloppyType := D1440;
        end;
        OkReadSector := ReadFloppySectors(cbDrive.Drive, A, FloppyType);
      end else begin
        SectorLength := BytesPerSector;
        OkReadSector := ReadSectors(A);
      end;
      if OkReadSector then begin
        BytesToGrid;
        B := A;
        StatusBar1.Panels[0].Text := Format('Sector #%D',[seFirstSector.Value]);
        bWriteSec.Enabled := True;
      end else begin
        StatusBar1.Panels[1].Text := 'Error Sector Read!';
        Application.MessageBox('Error Sector Read!',
           PChar('Disk '+cbDrive.Drive+':'),MB_OK + MB_DEFBUTTON1);
        cbDrive.Drive := OldDrive;
        bWriteSec.Enabled := False;
      end;
      StringGrid1.Col := 1;
      StringGrid1.Row := 1;
      StatusBar1.Panels[1].Text := '';
    end else begin
      StatusBar1.Panels[0].Text := Format('Sector #%D',[FirstSector]);
      StatusBar1.Panels[1].Text :=
       Format('Sector %D not found!',[seFirstSector.Value]);
      seFirstSector.Value := FirstSector;
    end;  (* num sectors on disk check *)
  end;  (* with DirDisk1 *)
  OldDrive := cbDrive.Drive;
end;

procedure TForm1.bWriteSecClick(Sender: TObject);
var Floppy: Boolean;
begin
  Floppy := UpCase(cbDrive.Drive) in ['A','B'];
  if Floppy then DirDisk1.WriteFloppySectors(cbDrive.Drive, B, FloppyType, False)
  else DirDisk1.WriteSectors(B);
end;

procedure TForm1.BytesToGrid;
var
  I, J, K: Integer;
  C: Char;
begin
  for i:=0 to (SectorLength div 16)-1 do begin  //rows
    StringGrid1.Cells[0,i+1] := IntToHex(i*16,4)+':';
    for j:=1 to 16 do begin  //columns
      K := A[j-1+16*i];
      StringGrid1.Cells[j,i+1] := IntToHex(K,2);
      C := Chr(K);
      if C<' ' then C := '.';
      StringGrid1.Cells[j+17,i+1] := C;
    end;
  end;
end;

procedure TForm1.StringGrid1KeyPress(Sender: TObject; var Key: Char);
begin
  if Key<>#8 then begin
    Key := UpCase(Key);
    if not (Key in ['0'..'9','A'..'F']) then Key := #0 else
    with StringGrid1 do
      if CellKeyPress>1 then begin
        if Length(Cells[Col,Row])>1 then begin  //2 digits to cell only
          if (Col+1+CharSellsCount)<ColCount then begin
            Col := Col+1;  //next cell
            CellKeyPress := 1;
          end else begin
            if (Row+1)<RowCount then begin
              Col := 1; Row := Row+1;  //next row
              CellKeyPress := 1;
            end else Key := #0;  //end of the grid!
          end;
        end;
      end else Inc(CellKeyPress);
  end;    (* if Key<>#8*)
  if Key<>#0 then StatusBar1.Panels[1].Text := 'Modified!';
end;

procedure TForm1.FormCreate(Sender: TObject);
var I: Integer;
begin
  with StringGrid1 do begin
    ColWidths[0] := 50;
    for i := 17 to 33 do begin
      TabStops[i] := False;
      ColWidths[i] := 11;
    end;    (* for *)
    for i := 1 to 16 do Cells[i,0] := IntToHex(i-1,2);
    for i := 18 to 33 do Cells[i,0] := IntToHex(i-18,1);
    Cells[0,0] := 'Offset';
    Form1.Width := GridWidth+32;
    bReadSecClick(Sender);
  end;    (* with *)
end;

procedure TForm1.OnFocusChange;
var
  I: Integer;
  C: Char;
begin
  I := ColOld-1+(RowOld-1)*16;
  with StringGrid1 do
    if Length(Cells[ColOld,RowOld])>2 then begin
      Cells[ColOld,RowOld] := IntToHex(A[I],2);
    end else
    try
      B[I] := StrToInt('$'+Cells[ColOld,RowOld]);
      C := Chr(B[I]);
      if C<' ' then C := '.';
      Cells[ColOld+17,RowOld] := C;
    except; end;
end;

procedure TForm1.StringGrid1Exit(Sender: TObject);
begin
  with StringGrid1 do
    if Col<16 then
      Col := Col+1
    else begin
      Col := 1; Row := Row+1;
    end;
  OnFocusChange;
end;

procedure TForm1.StringGrid1DrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var I: LongInt;
begin
  with StringGrid1.Canvas do
    if (Col>0) and (Row>0) then begin
      case Col of
        1..16: I := Col-1+(Row-1)*16;
        18..33: I := Col-18+(Row-1)*16;
      else I := -1;
      end;    (* case *)
      if I>=0 then
        if A[I]<>B[I] then begin
          Brush.Color := clYellow;
          FillRect(Rect);
          TextOut(Rect.Left+2,Rect.Top+2,StringGrid1.Cells[Col,Row]);
        end;
    end;
end;

procedure TForm1.StringGrid1SelectCell(Sender: TObject; Col, Row: Longint;
  var CanSelect: Boolean);
begin
  if Col>16 then CanSelect := False;
  CellKeyPress := 0;
  OnFocusChange;
end;

procedure TForm1.StringGrid1GetEditText(Sender: TObject; ACol,
  ARow: Longint; var Value: string);
var
  S: String;
  L: Integer;
begin
  with StringGrid1 do begin
    S := Cells[ColOld,RowOld];
    L := Length(S);
    case L of
      0: Cells[ColOld,RowOld] := '00';
      1: Cells[ColOld,RowOld] := '0'+S;
    end;
  end;    (* with *)
  ColOld := ACol; RowOld := ARow;
end;

procedure TForm1.cbDriveChange(Sender: TObject);
begin
  if Ord(cbDrive.Drive)>Ord('b') then begin
    DirDisk1.DriveName := cbDrive.Drive;
    if not DirDisk1.TrueDisk then begin
      cbDrive.Drive := DirDisk1.DriveName;
      Beep;
    end else begin
      bWriteSec.Enabled := False;
      bReadSecClick(Sender);
    end;
  end else begin
    bReadSecClick(Sender);
  end;
end;

procedure TForm1.seFirstSectorChange(Sender: TObject);
begin
  bWriteSec.Enabled := False;
end;

end.
